





IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.utils_time_to_zero') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
   DROP FUNCTION dbo.utils_time_to_zero
GO
CREATE FUNCTION dbo.utils_time_to_zero
(
   @date_with_time datetime
)
RETURNS datetime
AS
BEGIN
   RETURN convert(datetime, (SELECT convert(nchar(4), datepart(yyyy, @date_with_time)) +
      + '-' + convert(nchar(2), datepart(mm, @date_with_time)) +
      + '-' + convert(nchar(2), datepart(dd, @date_with_time))
      + ' 00:00:00'), 121);
END;
GO



IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.utils_get_weeks_list') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
   DROP FUNCTION dbo.utils_get_weeks_list
GO
CREATE FUNCTION dbo.utils_get_weeks_list
(
   @start_date datetime,
   @end_date datetime
)
RETURNS @week_list TABLE (week_num integer, start_date datetime)
AS
BEGIN
   DECLARE @i int;
   SELECT @i = 1;
   SELECT @start_date = dbo.utils_time_to_zero(@start_date);
   WHILE datediff(ww, @start_date, @end_date) >= 0
   BEGIN
      INSERT INTO @week_list(week_num, start_date)
         VALUES (@i, @start_date)

      SELECT @start_date = dateadd(ww, 1, @start_date),
             @i = @i + 1;
   END;
   RETURN;
END;
GO



IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.utils_get_days_list') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
   DROP FUNCTION dbo.utils_get_days_list
GO
CREATE FUNCTION dbo.utils_get_days_list
(
   @start_date datetime,
   @end_date datetime
)
RETURNS @days_list TABLE (day_num integer, start_date datetime)
AS
BEGIN
   DECLARE @i int;
   SELECT @i = 1;
   SELECT @start_date = dbo.utils_time_to_zero(@start_date);
   WHILE datediff(dd, @start_date, @end_date) >= 0
   BEGIN
      INSERT INTO @days_list(day_num, start_date)
         VALUES (@i, @start_date)

      SELECT @start_date = dateadd(dd, 1, @start_date),
             @i = @i + 1;
   END;
   RETURN;
END;
GO






IF EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sales_forecast_init') AND objectproperty(object_id, 'IsProcedure') = 1)
   DROP PROCEDURE dbo.sales_forecast_init
GO
CREATE PROCEDURE dbo.sales_forecast_init
   @first_week     datetime,
   @last_week      datetime,
   @first_week_ref datetime
AS
BEGIN
   SET NOCOUNT ON;

   SELECT @first_week     = dbo.utils_time_to_zero(@first_week),
          @last_week      = dbo.utils_time_to_zero(@last_week),
          @first_week_ref = dbo.utils_time_to_zero(@first_week_ref);

   DELETE FROM sales_forecast
      WHERE week BETWEEN @first_week AND @last_week;

   INSERT INTO sales_forecast
      (store_id, 
       product_id, 
       week, 
       quantity, 
       avg_price)
   SELECT s.store_id, 
          s.product_id, 
          w.start_date, 
          sum(s.quantity),
          avg(s.price)
      FROM sales s
           LEFT OUTER JOIN dbo.utils_get_weeks_list(@first_week, @last_week) w
              ON s.day BETWEEN dateadd(ww, datediff(ww, @first_week, w.start_date), @first_week_ref)
                       AND dateadd(ww, datediff(ww, @first_week, w.start_date) + 1, @first_week_ref)
      GROUP BY s.store_id,
               s.product_id, 
               w.start_date;
END;
GO