





IF EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sales_forecast_init') AND objectproperty(object_id, 'IsProcedure') = 1)
   DROP PROCEDURE dbo.sales_forecast_init
GO
CREATE PROCEDURE dbo.sales_forecast_init
   @first_week     datetime,
   @last_week      datetime,
   @first_week_ref datetime
AS
BEGIN
   SET NOCOUNT ON;

   SELECT @first_week     = dbo.utils_time_to_zero(@first_week),
          @last_week      = dbo.utils_time_to_zero(@last_week),
          @first_week_ref = dbo.utils_time_to_zero(@first_week_ref);

   DELETE FROM sales_forecast
      WHERE week BETWEEN @first_week AND @last_week;

   INSERT INTO sales_forecast
      (store_id, 
       product_id, 
       week, 
       quantity, 
       avg_price)
   SELECT s.store_id, 
          s.product_id, 
          w.start_date, 
          sum(s.quantity),
          avg(s.price)
      FROM sales s
           LEFT OUTER JOIN dbo.utils_get_weeks_list(@first_week, @last_week) w
              ON s.day BETWEEN dateadd(ww, datediff(ww, @first_week, w.start_date), @first_week_ref)
                       AND dateadd(ww, datediff(ww, @first_week, w.start_date) + 1, @first_week_ref)
      GROUP BY s.store_id,
               s.product_id, 
               w.start_date;
END;
GO