








IF EXISTS(SELECT * FROM sys.views WHERE object_id = object_id(N'dbo.rand2'))
   DROP VIEW dbo.rand2
GO
CREATE VIEW dbo.rand2 AS 
  SELECT rand(abs(convert(int, convert(varbinary, newid())))) AS rand_value;
GO








IF EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.test_sales_forecast_setup') AND objectproperty(object_id, 'IsProcedure') = 1)
   DROP PROCEDURE dbo.test_sales_forecast_setup
GO
CREATE PROCEDURE dbo.test_sales_forecast_setup
AS
BEGIN
   SET NOCOUNT ON;

   /* Fill test data */
   INSERT INTO products(id, reference, name)
   SELECT newid(), 'TEST#prod1', 'Product 1'
   UNION
   SELECT newid(), 'TEST#prod2', 'Product 2'
   UNION
   SELECT newid(), 'TEST#prod3', 'Product 3'


   INSERT INTO stores(id, reference, name)
   SELECT newid(), 'TEST#store1', 'Store1 Msk'
   UNION
   SELECT newid(), 'TEST#store2', 'Store2 SPb';


   INSERT INTO sales (store_id, product_id, [day], quantity, price)
   SELECT s.id, p.id, d.start_date,
          convert(int, ((SELECT rand_value FROM rand2) * 1000)),
          (SELECT rand_value FROM rand2) * 100.00
      FROM products p
           CROSS JOIN
           stores s
           CROSS JOIN
           dbo.utils_get_days_list(convert(datetime, '2007-01-08', 21), convert(datetime, '2007-02-19', 21)) d
      WHERE p.reference LIKE 'TEST#%' AND
            s.reference LIKE 'TEST#%'


   DECLARE
    @Val50 int,
    @Val51 int,
    @ValStr50 nvarchar(255),
    @GivenValStr50 nvarchar(255);
SELECT
    @Val50 = (SELECT count(*)
          FROM sales 
               INNER JOIN stores ON sales.store_id = stores.id
               INNER JOIN products ON sales.product_id = products.id
          WHERE stores.reference LIKE 'TEST#%' AND 
                products.reference LIKE 'TEST#%' AND 
                day BETWEEN convert(datetime, '2007-01-08', 21) AND convert(datetime, '2007-02-19', 21)
      ),
    @Val51 = (0);
IF (@Val50 IS NULL OR NOT(@Val50 > @Val51))
BEGIN
    SELECT @ValStr50 = convert(nvarchar(255), @Val50),
        @GivenValStr50 = convert(nvarchar(255), @Val51);
    raiserror('Error. Value "%s" is less or equals than "%s". Sales table has no data', 16, 1, @ValStr50, @GivenValStr50);
END;;

END;
GO


IF EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.test_sales_forecast_teardown') AND objectproperty(object_id, 'IsProcedure') = 1)
   DROP PROCEDURE dbo.test_sales_forecast_teardown
GO
CREATE PROCEDURE dbo.test_sales_forecast_teardown
AS
BEGIN
   SET NOCOUNT ON;

   DELETE FROM sales_forecast
      WHERE product_id IN (SELECT id FROM products WHERE reference LIKE 'TEST#%');

   DELETE FROM sales 
      WHERE product_id IN (SELECT id FROM products WHERE reference LIKE 'TEST#%');

   DELETE FROM products WHERE reference LIKE 'TEST#%';
   DELETE FROM stores WHERE reference LIKE 'TEST#%';
END;
GO


IF EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.test_sales_forecast_init') AND objectproperty(object_id, 'IsProcedure') = 1)
   DROP PROCEDURE dbo.test_sales_forecast_init
GO
CREATE PROCEDURE dbo.test_sales_forecast_init
AS
BEGIN
   SET NOCOUNT ON;

   DECLARE @first_week     datetime,
           @last_week      datetime,
           @first_week_ref datetime;
   SELECT 
      @first_week     = convert(datetime, '2008-01-07', 21),
      @last_week      = convert(datetime, '2008-02-18', 21),
      @first_week_ref = convert(datetime, '2007-01-08', 21);


   EXEC dbo.sales_forecast_init
      @first_week     = @first_week,
      @last_week      = @last_week,
      @first_week_ref = @first_week_ref;


   DECLARE
    @Val104 int,
    @Val105 int,
    @ValStr104 nvarchar(255),
    @GivenValStr104 nvarchar(255);
SELECT
    @Val104 = (SELECT count(*)
          FROM sales_forecast
               INNER JOIN stores ON sales_forecast.store_id = stores.id
               INNER JOIN products ON sales_forecast.product_id = products.id
          WHERE stores.reference LIKE 'TEST#%' AND 
                products.reference LIKE 'TEST#%' AND 
                week BETWEEN @first_week AND @last_week
      ),
    @Val105 = (0);
IF (@Val104 IS NULL OR NOT(@Val104 > @Val105))
BEGIN
    SELECT @ValStr104 = convert(nvarchar(255), @Val104),
        @GivenValStr104 = convert(nvarchar(255), @Val105);
    raiserror('Error. Value "%s" is less or equals than "%s". Sales forecasts has no data', 16, 1, @ValStr104, @GivenValStr104);
END;;

   /* Check calculated data on random week */
   DECLARE @store_id      uniqueidentifier,
           @product_id    uniqueidentifier,
           @week_to_check datetime,
           @ref_week      datetime;

   SELECT @store_id = id FROM stores WHERE reference = 'TEST#store1';
   SELECT @product_id = id FROM products WHERE reference = 'TEST#prod2';
   SELECT @week_to_check = 
      dateadd(ww, 
              datediff(ww, @first_week, @last_week) * (SELECT rand_value FROM rand2), 
              @first_week);
   SELECT @ref_week = dateadd(ww, datediff(ww, @first_week, @week_to_check), @first_week_ref);

   DECLARE
    @Val131 int,
    @Val132 int,
    @ValStr131 nvarchar(255),
    @GivenValStr131 nvarchar(255);
SELECT
    @Val131 = (SELECT quantity
          FROM sales_forecast
          WHERE store_id = @store_id AND 
                product_id = @product_id AND
                week = @week_to_check
      ),
    @Val132 = (SELECT SUM(quantity) 
          FROM sales 
          WHERE store_id = @store_id AND 
                product_id = @product_id AND
                [day] BETWEEN @ref_week AND dateadd(ww, 1, @ref_week)
      );
IF (@Val131 IS NULL OR NOT(@Val131 = @Val132))
BEGIN
    SELECT @ValStr131 = convert(nvarchar(255), @Val131),
        @GivenValStr131 = convert(nvarchar(255), @Val132);
    raiserror('Error. Value is "%s". Expected "%s". Invalid quantity', 16, 1, @ValStr131, @GivenValStr131);
END;;

   DECLARE
    @Val147 int,
    @Val148 int,
    @ValStr147 nvarchar(255),
    @GivenValStr147 nvarchar(255);
SELECT
    @Val147 = (SELECT avg_price
          FROM sales_forecast
          WHERE store_id = @store_id AND 
                product_id = @product_id AND
                week = @week_to_check
      ),
    @Val148 = (SELECT AVG(price) 
          FROM sales 
          WHERE store_id = @store_id AND 
                product_id = @product_id AND
                [day] BETWEEN @ref_week AND dateadd(ww, 1, @ref_week)
      );
IF (@Val147 IS NULL OR NOT(@Val147 = @Val148))
BEGIN
    SELECT @ValStr147 = convert(nvarchar(255), @Val147),
        @GivenValStr147 = convert(nvarchar(255), @Val148);
    raiserror('Error. Value is "%s". Expected "%s". Invalid average price', 16, 1, @ValStr147, @GivenValStr147);
END;;

END;
GO


IF EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.test_sales_forecast_all') AND objectproperty(object_id, 'IsProcedure') = 1)
   DROP PROCEDURE dbo.test_sales_forecast_all
GO
CREATE PROCEDURE dbo.test_sales_forecast_all
AS
BEGIN
   SET NOCOUNT ON;
   EXEC dbo.test_sales_forecast_setup;
   EXEC dbo.test_sales_forecast_init;
   EXEC dbo.test_sales_forecast_teardown;
END;
GO
